from importlib.metadata import version
from pathlib import Path
from secrets import compare_digest as compare_digest_secure
from typing import Any, Awaitable, Callable, cast

from fastapi import Depends, FastAPI, HTTPException, WebSocket, WebSocketException
from fastapi.middleware.gzip import GZipMiddleware
from fastapi.requests import HTTPConnection, Request
from fastapi.responses import JSONResponse
from fastapi.security import HTTPBasic, HTTPBasicCredentials
from fastapi.staticfiles import StaticFiles
from starlette.status import WS_1008_POLICY_VIOLATION

from ..exceptions import PluginGoCQException
from ..log import AccessLogFilter
from ..plugin_config import config as plugin_config
from .api import router as api_router

DIST_PATH = Path(__file__).parent / "dist"

if not DIST_PATH.is_dir():
    raise FileNotFoundError("WebUI dist directory not found")


class WSCompatibleHTTPBasic(HTTPBasic):
    async def __call__(self, request: HTTPConnection):
        try:
            return await super().__call__(cast(Request, request))
        except HTTPException as e:
            if not isinstance(request, WebSocket):
                raise
            await request.accept()
            raise WebSocketException(
                code=WS_1008_POLICY_VIOLATION, reason=e.detail
            ) from None


async def security_dependency(
    credentials: HTTPBasicCredentials = Depends(WSCompatibleHTTPBasic()),
):
    assert plugin_config.WEBUI_USERNAME and plugin_config.WEBUI_PASSWORD
    if not (
        compare_digest_secure(credentials.username, plugin_config.WEBUI_USERNAME)
        and compare_digest_secure(credentials.password, plugin_config.WEBUI_PASSWORD)
    ):
        raise HTTPException(
            status_code=401,
            detail="Unauthorized credentials",
            headers={"WWW-Authenticate": "Basic"},
        )
    return credentials


app = FastAPI(
    title="nonebot-plugin-gocqhttpQQ",
    description="go-cqhttp process manager API",
    dependencies=(
        [Depends(security_dependency)]
        if plugin_config.WEBUI_PASSWORD and plugin_config.WEBUI_USERNAME
        else []
    ),
    version=version("nonebot-plugin-gocqhttpQQ"),
)


@app.middleware("http")
async def add_filterable_log(
    request: Request, call_next: Callable[[Request], Awaitable[Any]]
):
    AccessLogFilter.filterable_paths.add(request.scope["path"])
    return await call_next(request)


@app.exception_handler(PluginGoCQException)
async def handle_plugin_exception(request: Request, exc: PluginGoCQException):
    return JSONResponse(content={"detail": exc.message}, status_code=exc.code)


app.add_middleware(GZipMiddleware, minimum_size=1024)

app.include_router(api_router, prefix="/api")

app.mount("/", StaticFiles(directory=DIST_PATH, html=True), name="frontend")
